import builtins
import copy
import datetime
import string
import uuid

from jinja2 import Environment, FileSystemLoader


def base_context():
    defaults = [_ for _ in dir(builtins)
                if _[0] in string.ascii_lowercase and
                _ not in ['copyright', 'credits']
                ]
    attrs = [getattr(builtins, _)
             for _ in defaults if _ not in ['copyright', 'credits']]

    builtins_dict = dict(zip(defaults, attrs))
    return copy.deepcopy(builtins_dict)


def generate(file_in_templates, out_path, template_dir='templates', assets_path_append='', **kwargs):
    """
    Generates necessary file(s)
    :param file_in_templates: template to work with
    :param out_path: output path to save the generated file to
    :param template_dir: templates directory
    :param assets_path_append:
    :param kwargs: variables
    :return: None
    """

    file_loader = FileSystemLoader(template_dir)
    env = Environment(loader=file_loader)
    template = env.get_template(file_in_templates)

    build_id = str(uuid.uuid4())  # to be used

    output = template.render(kwargs, year=datetime.datetime.now().year,
                             build_id=build_id, assets_path_append=assets_path_append)
    with open(out_path, 'w+', encoding="utf8") as f:
        f.write(output)



OUTPUT_FOLDER = 'dist/'
info = {
    "social": {
        "facebook": "",
        "twitter": "https://twitter.com/osdotsystem",
        "instagram": "",
        "github": ""
    },
    "nav": [
        ["Main Page", "index.html", "page_1"],
        ["Page 2", "index2.html", "page_2"],
        ["Page 3", "index3.html", "page_3"],
    ]
}



# https://github.com/pymug/website-AV19-AV20

import sys
from os.path import join

import settings
from flask import Flask
from jamstack.api.template import base_context, generate
from livereload import Server

context = base_context()
context.update({
    "info": settings.info
})


def main(args):
    def gen():
        generate('index.html', join(
            settings.OUTPUT_FOLDER, 'index.html'), **context)
        generate('index2.html', join(
            settings.OUTPUT_FOLDER, 'index2.html'), **context)
        generate('index3.html', join(
            settings.OUTPUT_FOLDER, 'index3.html'), **context)

    if len(args) > 1 and args[1] == '--server':
        app = Flask(__name__)

        # remember to use DEBUG mode for templates auto reload
        # https://github.com/lepture/python-livereload/issues/144
        app.debug = True
        server = Server(app.wsgi_app)

        # run a shell command
        # server.watch('.', 'make static')

        # run a function

        server.watch('.', gen, delay=5)
        server.watch('*.py')

        # output stdout into a file
        # server.watch('style.less', shell('lessc style.less', output='style.css'))

        server.serve()
    else:
        gen()


if __name__ == '__main__':
    main(sys.argv)



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject


python setup.py publish to publish

"""

import os
import sys

from jamstack import __version__  # thanks gunicorn
# Always prefer setuptools over distutils
from setuptools import setup

# from setuptools import find_packages


here = os.path.abspath(os.path.dirname(__file__))

if sys.argv[-1] == "publish":  # requests
    os.system("python setup.py sdist")  # bdist_wheel
    os.system("twine upload dist/* --skip-existing")
    sys.exit()

# Get the long description from the README file
with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()
setup(
    name="jamstack",  # Required
    version=__version__,  # Required
    description="Jamstack in Python",  # Optional
    long_description=long_description,  # Optional
    long_description_content_type="text/markdown",  # Optional (see note above)
    url="https://github.com/Abdur-RahmaanJ/jamstack",  # Optional
    author="Abdur-Rahmaan Janhangeer",  # Optional
    author_email="arj.python@gmail.com",  # Optional
    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        # 'Topic :: Weather',
        # Pick your license as you wish
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        # These classifiers are *not* checked by 'pip install'. See instead
        # 'python_requires' below.
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="jamstack static",  # Optional
    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    # packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required
    packages=["jamstack"],
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=open(
        os.path.join(here, "requirements.txt"), encoding="utf-8"
    )
    .read()
    .split("\n"),  # Optional
    project_urls={  # Optional
        "Bug Reports": "https://github.com/Abdur-RahmaanJ/jamstack/issues",
        "Source": "https://github.com/Abdur-RahmaanJ/jamstack/",
    },
    entry_points={"console_scripts": ["jamstack=jamstack.__main__:main"]},
)



import os
import shutil
import uuid

# from werkzeug.utils import secure_filename


def trycopytree(source, dest, verbose=False, dirs_exist_ok=False):
    """
    Recursive copy of folder

    Parameters
    ----------
    source: str
        source folder path
    dest: str
        destination folder path

    Returns
    -------
    None
    """
    try:
        shutil.copytree(source, dest, dirs_exist_ok=dirs_exist_ok)
        if verbose:
            print("done copying {} to {}".format(source, dest))
    except Exception as e:
        print(e)


def trycopy(source, dest, verbose=False):
    """
    Non-ecursive copy of folder

    Parameters
    ----------
    source: str
        source folder path
    dest: str
        destination folder path

    Returns
    -------
    None
    """
    try:
        shutil.copy(source, dest)
        if verbose:
            print("done copying {} to {}".format(source, dest))
    except Exception as e:
        print(e)


def trymkdir(path, verbose=False):
    """
    Creates dir at destination

    Parameters
    ----------
    path: str
        path with folder already in

    Returns
    -------
    None
    """
    try:
        os.mkdir(path)
        if verbose:
            print("created dir at", path)
    except Exception as e:
        print(e)


def trymkfile(path, content, verbose=False):
    """
    Creates file

    Parameters
    ----------
    path: str
        path to create file with filename included
    content: str
        file content

    Returns
    -------
    None
    """
    try:
        with open(path, "w+") as f:
            f.write(content)
        if verbose:
            print("file created at {}".format(path))
            print("with content:")
            print(content)
    except Exception as e:
        print(e)


def absdiroffile(filepath):
    """
    Gives absolute directory of file, normally expects __file__ as
    param

    Parameters
    ----------
    filepath: str
        path of file

    Returns
    -------
    str
        Absolute dir path of file
    """
    return os.path.dirname(os.path.abspath(filepath))


def get_folders(path):
    dirs = [d for d in os.listdir(
        path) if os.path.isdir(os.path.join(path, d))]
    return dirs


def unique_filename(fname):
    prepended = str(uuid.uuid4()).replace("-", "_")[:10]
    return "{}_{}".format(prepended, fname)


def delete_file(path):
    os.remove(path)


# def unique_sec_filename(filename):
#     return unique_filename(secure_filename(filename))



import os
from pathlib import Path

import click
from jamstack.api.file import trycopytree

package_folder = Path(__file__).parent.absolute()
sites_path = os.path.join(package_folder, 'sites')


@click.group(help="<Jamstack cli/>")
def cli():
    pass


@click.command(help="<Empty, plain repo />")
@click.argument('project_name')
@click.option('--existing/--not-existing', default=False)
def plain(project_name, existing):
    path = '.'

    dirs_exist_ok = False
    if existing is True:
        dirs_exist_ok = True
    trycopytree(
        os.path.join(sites_path, 'plain'),
        os.path.join(path, project_name),
        dirs_exist_ok=dirs_exist_ok
    )


@click.command(help="<Repo by templates />")
@click.argument('template')
@click.argument('project_name')
@click.option('--existing/--not-existing', default=False)
def t(template, project_name, existing):
    path = '.'
    templates = {'massively': 'html5up/massively',
                 'phantom': 'html5up/phantom'}

    # namespace = template.split('/')[0]
    # project = template.split('/')[1]

    dirs_exist_ok = False
    if existing is True:
        dirs_exist_ok = True
    trycopytree(
        os.path.join(sites_path, templates[template]),
        os.path.join(path, project_name),
        dirs_exist_ok=dirs_exist_ok
    )


cli.add_command(plain)
cli.add_command(t)


def main():
    cli()


if __name__ == "__main__":
    cli()
